package java.nio;

import sun.misc.JavaNioAccess;
import sun.misc.JavaLangRefAccess;
import sun.misc.SharedSecrets;
import sun.misc.VM;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.util.concurrent.atomic.AtomicLong;
import sun.misc.Unsafe;

class Bits
{
    private static final Unsafe unsafe;
    private static final ByteOrder byteOrder;
    private static int pageSize;
    private static boolean unaligned;
    private static boolean unalignedKnown;
    private static volatile long maxMemory;
    private static final AtomicLong reservedMemory;
    private static final AtomicLong totalCapacity;
    private static final AtomicLong count;
    private static volatile boolean memoryLimitSet;
    private static final int MAX_SLEEPS = 9;
    static final int JNI_COPY_TO_ARRAY_THRESHOLD = 6;
    static final int JNI_COPY_FROM_ARRAY_THRESHOLD = 6;
    static final long UNSAFE_COPY_THRESHOLD = 1048576L;
    
    private Bits() {
    }
    
    static short swap(short i) {
        return Short.reverseBytes(i);
    }
    
    static char swap(char ch) {
        return Character.reverseBytes(ch);
    }
    
    static int swap(int i) {
        return Integer.reverseBytes(i);
    }
    
    static long swap(long i) {
        return Long.reverseBytes(i);
    }
    
    private static char makeChar(byte b, byte b2) {
        return (char)(b << 8 | (b2 & 0xFF));
    }
    
    static char getCharL(ByteBuffer byteBuffer, int n) {
        return makeChar(byteBuffer._get(n + 1), byteBuffer._get(n));
    }
    
    static char getCharL(long n) {
        return makeChar(_get(n + 1L), _get(n));
    }
    
    static char getCharB(ByteBuffer byteBuffer, int n) {
        return makeChar(byteBuffer._get(n), byteBuffer._get(n + 1));
    }
    
    static char getCharB(long n) {
        return makeChar(_get(n), _get(n + 1L));
    }
    
    static char getChar(ByteBuffer byteBuffer, int n, boolean b) {
        return b ? getCharB(byteBuffer, n) : getCharL(byteBuffer, n);
    }
    
    static char getChar(long n, boolean b) {
        return b ? getCharB(n) : getCharL(n);
    }
    
    private static byte char1(char c) {
        return (byte)(c >> 8);
    }
    
    private static byte char0(char c) {
        return (byte)c;
    }
    
    static void putCharL(ByteBuffer byteBuffer, int n, char c) {
        byteBuffer._put(n, char0(c));
        byteBuffer._put(n + 1, char1(c));
    }
    
    static void putCharL(long n, char c) {
        _put(n, char0(c));
        _put(n + 1L, char1(c));
    }
    
    static void putCharB(ByteBuffer byteBuffer, int n, char c) {
        byteBuffer._put(n, char1(c));
        byteBuffer._put(n + 1, char0(c));
    }
    
    static void putCharB(long n, char c) {
        _put(n, char1(c));
        _put(n + 1L, char0(c));
    }
    
    static void putChar(ByteBuffer byteBuffer, int n, char c, boolean b) {
        if (b) {
            putCharB(byteBuffer, n, c);
        }
        else {
            putCharL(byteBuffer, n, c);
        }
    }
    
    static void putChar(long n, char c, boolean b) {
        if (b) {
            putCharB(n, c);
        }
        else {
            putCharL(n, c);
        }
    }
    
    private static short makeShort(byte b, byte b2) {
        return (short)(b << 8 | (b2 & 0xFF));
    }
    
    static short getShortL(ByteBuffer byteBuffer, int n) {
        return makeShort(byteBuffer._get(n + 1), byteBuffer._get(n));
    }
    
    static short getShortL(long n) {
        return makeShort(_get(n + 1L), _get(n));
    }
    
    static short getShortB(ByteBuffer byteBuffer, int n) {
        return makeShort(byteBuffer._get(n), byteBuffer._get(n + 1));
    }
    
    static short getShortB(long n) {
        return makeShort(_get(n), _get(n + 1L));
    }
    
    static short getShort(ByteBuffer byteBuffer, int n, boolean b) {
        return b ? getShortB(byteBuffer, n) : getShortL(byteBuffer, n);
    }
    
    static short getShort(long n, boolean b) {
        return b ? getShortB(n) : getShortL(n);
    }
    
    private static byte short1(short n) {
        return (byte)(n >> 8);
    }
    
    private static byte short0(short n) {
        return (byte)n;
    }
    
    static void putShortL(ByteBuffer byteBuffer, int n, short n2) {
        byteBuffer._put(n, short0(n2));
        byteBuffer._put(n + 1, short1(n2));
    }
    
    static void putShortL(long n, short n2) {
        _put(n, short0(n2));
        _put(n + 1L, short1(n2));
    }
    
    static void putShortB(ByteBuffer byteBuffer, int n, short n2) {
        byteBuffer._put(n, short1(n2));
        byteBuffer._put(n + 1, short0(n2));
    }
    
    static void putShortB(long n, short n2) {
        _put(n, short1(n2));
        _put(n + 1L, short0(n2));
    }
    
    static void putShort(ByteBuffer byteBuffer, int n, short n2, boolean b) {
        if (b) {
            putShortB(byteBuffer, n, n2);
        }
        else {
            putShortL(byteBuffer, n, n2);
        }
    }
    
    static void putShort(long n, short n2, boolean b) {
        if (b) {
            putShortB(n, n2);
        }
        else {
            putShortL(n, n2);
        }
    }
    
    private static int makeInt(byte b, byte b2, byte b3, byte b4) {
        return b << 24 | (b2 & 0xFF) << 16 | (b3 & 0xFF) << 8 | (b4 & 0xFF);
    }
    
    static int getIntL(ByteBuffer byteBuffer, int n) {
        return makeInt(byteBuffer._get(n + 3), byteBuffer._get(n + 2), byteBuffer._get(n + 1), byteBuffer._get(n));
    }
    
    static int getIntL(long n) {
        return makeInt(_get(n + 3L), _get(n + 2L), _get(n + 1L), _get(n));
    }
    
    static int getIntB(ByteBuffer byteBuffer, int n) {
        return makeInt(byteBuffer._get(n), byteBuffer._get(n + 1), byteBuffer._get(n + 2), byteBuffer._get(n + 3));
    }
    
    static int getIntB(long n) {
        return makeInt(_get(n), _get(n + 1L), _get(n + 2L), _get(n + 3L));
    }
    
    static int getInt(ByteBuffer byteBuffer, int n, boolean b) {
        return b ? getIntB(byteBuffer, n) : getIntL(byteBuffer, n);
    }
    
    static int getInt(long n, boolean b) {
        return b ? getIntB(n) : getIntL(n);
    }
    
    private static byte int3(int n) {
        return (byte)(n >> 24);
    }
    
    private static byte int2(int n) {
        return (byte)(n >> 16);
    }
    
    private static byte int1(int n) {
        return (byte)(n >> 8);
    }
    
    private static byte int0(int n) {
        return (byte)n;
    }
    
    static void putIntL(ByteBuffer byteBuffer, int n, int n2) {
        byteBuffer._put(n + 3, int3(n2));
        byteBuffer._put(n + 2, int2(n2));
        byteBuffer._put(n + 1, int1(n2));
        byteBuffer._put(n, int0(n2));
    }
    
    static void putIntL(long n, int n2) {
        _put(n + 3L, int3(n2));
        _put(n + 2L, int2(n2));
        _put(n + 1L, int1(n2));
        _put(n, int0(n2));
    }
    
    static void putIntB(ByteBuffer byteBuffer, int n, int n2) {
        byteBuffer._put(n, int3(n2));
        byteBuffer._put(n + 1, int2(n2));
        byteBuffer._put(n + 2, int1(n2));
        byteBuffer._put(n + 3, int0(n2));
    }
    
    static void putIntB(long n, int n2) {
        _put(n, int3(n2));
        _put(n + 1L, int2(n2));
        _put(n + 2L, int1(n2));
        _put(n + 3L, int0(n2));
    }
    
    static void putInt(ByteBuffer byteBuffer, int n, int n2, boolean b) {
        if (b) {
            putIntB(byteBuffer, n, n2);
        }
        else {
            putIntL(byteBuffer, n, n2);
        }
    }
    
    static void putInt(long n, int n2, boolean b) {
        if (b) {
            putIntB(n, n2);
        }
        else {
            putIntL(n, n2);
        }
    }
    
    private static long makeLong(byte b, byte b2, byte b3, byte b4, byte b5, byte b6, byte b7, byte b8) {
        return (long)b << 56 | ((long)b2 & 0xFFL) << 48 | ((long)b3 & 0xFFL) << 40 | ((long)b4 & 0xFFL) << 32 | ((long)b5 & 0xFFL) << 24 | ((long)b6 & 0xFFL) << 16 | ((long)b7 & 0xFFL) << 8 | ((long)b8 & 0xFFL);
    }
    
    static long getLongL(ByteBuffer byteBuffer, int n) {
        return makeLong(byteBuffer._get(n + 7), byteBuffer._get(n + 6), byteBuffer._get(n + 5), byteBuffer._get(n + 4), byteBuffer._get(n + 3), byteBuffer._get(n + 2), byteBuffer._get(n + 1), byteBuffer._get(n));
    }
    
    static long getLongL(long n) {
        return makeLong(_get(n + 7L), _get(n + 6L), _get(n + 5L), _get(n + 4L), _get(n + 3L), _get(n + 2L), _get(n + 1L), _get(n));
    }
    
    static long getLongB(ByteBuffer byteBuffer, int n) {
        return makeLong(byteBuffer._get(n), byteBuffer._get(n + 1), byteBuffer._get(n + 2), byteBuffer._get(n + 3), byteBuffer._get(n + 4), byteBuffer._get(n + 5), byteBuffer._get(n + 6), byteBuffer._get(n + 7));
    }
    
    static long getLongB(long n) {
        return makeLong(_get(n), _get(n + 1L), _get(n + 2L), _get(n + 3L), _get(n + 4L), _get(n + 5L), _get(n + 6L), _get(n + 7L));
    }
    
    static long getLong(ByteBuffer byteBuffer, int n, boolean b) {
        return b ? getLongB(byteBuffer, n) : getLongL(byteBuffer, n);
    }
    
    static long getLong(long n, boolean b) {
        return b ? getLongB(n) : getLongL(n);
    }
    
    private static byte long7(long n) {
        return (byte)(int)(n >> 56);
    }
    
    private static byte long6(long n) {
        return (byte)(int)(n >> 48);
    }
    
    private static byte long5(long n) {
        return (byte)(int)(n >> 40);
    }
    
    private static byte long4(long n) {
        return (byte)(int)(n >> 32);
    }
    
    private static byte long3(long n) {
        return (byte)(int)(n >> 24);
    }
    
    private static byte long2(long n) {
        return (byte)(int)(n >> 16);
    }
    
    private static byte long1(long n) {
        return (byte)(int)(n >> 8);
    }
    
    private static byte long0(long n) {
        return (byte)(int)n;
    }
    
    static void putLongL(ByteBuffer byteBuffer, int n, long n2) {
        byteBuffer._put(n + 7, long7(n2));
        byteBuffer._put(n + 6, long6(n2));
        byteBuffer._put(n + 5, long5(n2));
        byteBuffer._put(n + 4, long4(n2));
        byteBuffer._put(n + 3, long3(n2));
        byteBuffer._put(n + 2, long2(n2));
        byteBuffer._put(n + 1, long1(n2));
        byteBuffer._put(n, long0(n2));
    }
    
    static void putLongL(long n, long n2) {
        _put(n + 7L, long7(n2));
        _put(n + 6L, long6(n2));
        _put(n + 5L, long5(n2));
        _put(n + 4L, long4(n2));
        _put(n + 3L, long3(n2));
        _put(n + 2L, long2(n2));
        _put(n + 1L, long1(n2));
        _put(n, long0(n2));
    }
    
    static void putLongB(ByteBuffer byteBuffer, int n, long n2) {
        byteBuffer._put(n, long7(n2));
        byteBuffer._put(n + 1, long6(n2));
        byteBuffer._put(n + 2, long5(n2));
        byteBuffer._put(n + 3, long4(n2));
        byteBuffer._put(n + 4, long3(n2));
        byteBuffer._put(n + 5, long2(n2));
        byteBuffer._put(n + 6, long1(n2));
        byteBuffer._put(n + 7, long0(n2));
    }
    
    static void putLongB(long n, long n2) {
        _put(n, long7(n2));
        _put(n + 1L, long6(n2));
        _put(n + 2L, long5(n2));
        _put(n + 3L, long4(n2));
        _put(n + 4L, long3(n2));
        _put(n + 5L, long2(n2));
        _put(n + 6L, long1(n2));
        _put(n + 7L, long0(n2));
    }
    
    static void putLong(ByteBuffer byteBuffer, int n, long n2, boolean b) {
        if (b) {
            putLongB(byteBuffer, n, n2);
        }
        else {
            putLongL(byteBuffer, n, n2);
        }
    }
    
    static void putLong(long n, long n2, boolean b) {
        if (b) {
            putLongB(n, n2);
        }
        else {
            putLongL(n, n2);
        }
    }
    
    static float getFloatL(ByteBuffer byteBuffer, int n) {
        return Float.intBitsToFloat(getIntL(byteBuffer, n));
    }
    
    static float getFloatL(long n) {
        return Float.intBitsToFloat(getIntL(n));
    }
    
    static float getFloatB(ByteBuffer byteBuffer, int n) {
        return Float.intBitsToFloat(getIntB(byteBuffer, n));
    }
    
    static float getFloatB(long n) {
        return Float.intBitsToFloat(getIntB(n));
    }
    
    static float getFloat(ByteBuffer byteBuffer, int n, boolean b) {
        return b ? getFloatB(byteBuffer, n) : getFloatL(byteBuffer, n);
    }
    
    static float getFloat(long n, boolean b) {
        return b ? getFloatB(n) : getFloatL(n);
    }
    
    static void putFloatL(ByteBuffer byteBuffer, int n, float n2) {
        putIntL(byteBuffer, n, Float.floatToRawIntBits(n2));
    }
    
    static void putFloatL(long n, float n2) {
        putIntL(n, Float.floatToRawIntBits(n2));
    }
    
    static void putFloatB(ByteBuffer byteBuffer, int n, float n2) {
        putIntB(byteBuffer, n, Float.floatToRawIntBits(n2));
    }
    
    static void putFloatB(long n, float n2) {
        putIntB(n, Float.floatToRawIntBits(n2));
    }
    
    static void putFloat(ByteBuffer byteBuffer, int n, float n2, boolean b) {
        if (b) {
            putFloatB(byteBuffer, n, n2);
        }
        else {
            putFloatL(byteBuffer, n, n2);
        }
    }
    
    static void putFloat(long n, float n2, boolean b) {
        if (b) {
            putFloatB(n, n2);
        }
        else {
            putFloatL(n, n2);
        }
    }
    
    static double getDoubleL(ByteBuffer byteBuffer, int n) {
        return Double.longBitsToDouble(getLongL(byteBuffer, n));
    }
    
    static double getDoubleL(long n) {
        return Double.longBitsToDouble(getLongL(n));
    }
    
    static double getDoubleB(ByteBuffer byteBuffer, int n) {
        return Double.longBitsToDouble(getLongB(byteBuffer, n));
    }
    
    static double getDoubleB(long n) {
        return Double.longBitsToDouble(getLongB(n));
    }
    
    static double getDouble(ByteBuffer byteBuffer, int n, boolean b) {
        return b ? getDoubleB(byteBuffer, n) : getDoubleL(byteBuffer, n);
    }
    
    static double getDouble(long n, boolean b) {
        return b ? getDoubleB(n) : getDoubleL(n);
    }
    
    static void putDoubleL(ByteBuffer byteBuffer, int n, double n2) {
        putLongL(byteBuffer, n, Double.doubleToRawLongBits(n2));
    }
    
    static void putDoubleL(long n, double n2) {
        putLongL(n, Double.doubleToRawLongBits(n2));
    }
    
    static void putDoubleB(ByteBuffer byteBuffer, int n, double n2) {
        putLongB(byteBuffer, n, Double.doubleToRawLongBits(n2));
    }
    
    static void putDoubleB(long n, double n2) {
        putLongB(n, Double.doubleToRawLongBits(n2));
    }
    
    static void putDouble(ByteBuffer byteBuffer, int n, double n2, boolean b) {
        if (b) {
            putDoubleB(byteBuffer, n, n2);
        }
        else {
            putDoubleL(byteBuffer, n, n2);
        }
    }
    
    static void putDouble(long n, double n2, boolean b) {
        if (b) {
            putDoubleB(n, n2);
        }
        else {
            putDoubleL(n, n2);
        }
    }
    
    private static byte _get(long address) {
        return Bits.unsafe.getByte(address);
    }
    
    private static void _put(long address, byte x) {
        Bits.unsafe.putByte(address, x);
    }
    
    static Unsafe unsafe() {
        return Bits.unsafe;
    }
    
    static ByteOrder byteOrder() {
        if (Bits.byteOrder == null) {
            throw new Error("Unknown byte order");
        }
        return Bits.byteOrder;
    }
    
    static int pageSize() {
        if (Bits.pageSize == -1) {
            Bits.pageSize = unsafe().pageSize();
        }
        return Bits.pageSize;
    }
    
    static int pageCount(long n) {
        return (int)(n + (long)pageSize() - 1L) / pageSize();
    }
    
    static boolean unaligned() {
        if (Bits.unalignedKnown) {
            return Bits.unaligned;
        }
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("os.arch"));
        Bits.unaligned = (s.equals("i386") || s.equals("x86") || s.equals("amd64") || s.equals("x86_64") || s.equals("ppc64") || s.equals("ppc64le") || s.equals("aarch64"));
        Bits.unalignedKnown = true;
        return Bits.unaligned;
    }
    
    static void reserveMemory(long n, int n2) {
        if (!Bits.memoryLimitSet && VM.isBooted()) {
            Bits.maxMemory = VM.maxDirectMemory();
            Bits.memoryLimitSet = true;
        }
        if (tryReserveMemory(n, n2)) {
            return;
        }
        JavaLangRefAccess javaLangRefAccess = SharedSecrets.getJavaLangRefAccess();
        while (javaLangRefAccess.tryHandlePendingReference()) {
            if (!tryReserveMemory(n, n2)) {
                continue;
            }
            return;
        }
        System.gc();
        int n3 = 0;
        try {
            long millis = 1L;
            int n4 = 0;
            while (!tryReserveMemory(n, n2)) {
                if (n4 >= 9) {
                    throw new OutOfMemoryError("Direct buffer memory");
                }
                if (javaLangRefAccess.tryHandlePendingReference()) {
                    continue;
                }
                try {
                    Thread.sleep(millis);
                    millis <<= 1;
                    ++n4;
                }
                catch (InterruptedException ex) {
                    n3 = 1;
                }
            }
            return;
        }
        finally {
            if (n3 != 0) {
                Thread.currentThread().interrupt();
            }
        }
    }
    
    private static boolean tryReserveMemory(long delta, int n) {
        long value;
        while ((long)n <= Bits.maxMemory - (value = Bits.totalCapacity.get())) {
            if (!Bits.totalCapacity.compareAndSet(value, value + (long)n)) {
                continue;
            }
            Bits.reservedMemory.addAndGet(delta);
            Bits.count.incrementAndGet();
            return true;
        }
        return false;
    }
    
    static void unreserveMemory(long n, int n2) {
        long decrementAndGet = Bits.count.decrementAndGet();
        long addAndGet = Bits.reservedMemory.addAndGet(-n);
        long addAndGet2 = Bits.totalCapacity.addAndGet((long)(-n2));
        assert decrementAndGet >= 0L && addAndGet >= 0L && addAndGet2 >= 0L;
        return;
    }
    
    static void copyFromArray(Object srcBase, long n, long n2, long destOffset, long bytes) {
        for (long srcOffset = n + n2; bytes > 0L; bytes -= bytes, srcOffset += bytes, destOffset += bytes) {
            bytes = ((bytes > 1048576L) ? 1048576L : bytes);
            Bits.unsafe.copyMemory(srcBase, srcOffset, null, destOffset, bytes);
        }
    }
    
    static void copyToArray(long srcOffset, Object destBase, long n, long n2, long bytes) {
        for (long destOffset = n + n2; bytes > 0L; bytes -= bytes, srcOffset += bytes, destOffset += bytes) {
            bytes = ((bytes > 1048576L) ? 1048576L : bytes);
            Bits.unsafe.copyMemory(null, srcOffset, destBase, destOffset, bytes);
        }
    }
    
    static void copyFromCharArray(Object o, long n, long n2, long n3) {
        copySwapMemory(o, (long)Bits.unsafe.arrayBaseOffset(o.getClass()) + n, null, n2, n3, 2L);
    }
    
    static void copyToCharArray(long n, Object o, long n2, long n3) {
        copySwapMemory(null, n, o, (long)Bits.unsafe.arrayBaseOffset(o.getClass()) + n2, n3, 2L);
    }
    
    static void copyFromShortArray(Object o, long n, long n2, long n3) {
        copySwapMemory(o, (long)Bits.unsafe.arrayBaseOffset(o.getClass()) + n, null, n2, n3, 2L);
    }
    
    static void copyToShortArray(long n, Object o, long n2, long n3) {
        copySwapMemory(null, n, o, (long)Bits.unsafe.arrayBaseOffset(o.getClass()) + n2, n3, 2L);
    }
    
    static void copyFromIntArray(Object o, long n, long n2, long n3) {
        copySwapMemory(o, (long)Bits.unsafe.arrayBaseOffset(o.getClass()) + n, null, n2, n3, 4L);
    }
    
    static void copyToIntArray(long n, Object o, long n2, long n3) {
        copySwapMemory(null, n, o, (long)Bits.unsafe.arrayBaseOffset(o.getClass()) + n2, n3, 4L);
    }
    
    static void copyFromLongArray(Object o, long n, long n2, long n3) {
        copySwapMemory(o, (long)Bits.unsafe.arrayBaseOffset(o.getClass()) + n, null, n2, n3, 8L);
    }
    
    static void copyToLongArray(long n, Object o, long n2, long n3) {
        copySwapMemory(null, n, o, (long)Bits.unsafe.arrayBaseOffset(o.getClass()) + n2, n3, 8L);
    }
    
    private static boolean isPrimitiveArray(Class<?> clazz) {
        Class<Object> componentType = clazz.getComponentType();
        return componentType != null && componentType.isPrimitive();
    }
    
    private static native void copySwapMemory0(Object p0, long p1, Object p2, long p3, long p4, long p5);
    
    private static void copySwapMemory(Object o, long n, Object o2, long n2, long n3, long n4) {
        if (n3 < 0L) {
            throw new IllegalArgumentException();
        }
        if (n4 != 2L && n4 != 4L && n4 != 8L) {
            throw new IllegalArgumentException();
        }
        if (n3 % n4 != 0L) {
            throw new IllegalArgumentException();
        }
        if ((o == null && n == 0L) || (o2 == null && n2 == 0L)) {
            throw new NullPointerException();
        }
        if (o != null && (n < 0L || !isPrimitiveArray(o.getClass()))) {
            throw new IllegalArgumentException();
        }
        if (o2 != null && (n2 < 0L || !isPrimitiveArray(o2.getClass()))) {
            throw new IllegalArgumentException();
        }
        if (Bits.unsafe.addressSize() == 4 && (n3 >>> 32 != 0L || n >>> 32 != 0L || n2 >>> 32 != 0L)) {
            throw new IllegalArgumentException();
        }
        if (n3 == 0L) {
            return;
        }
        copySwapMemory0(o, n, o2, n2, n3, n4);
        return;
    }
    
    static {
        unsafe = Unsafe.getUnsafe();
        long allocateMemory = Bits.unsafe.allocateMemory(8L);
        try {
            Bits.unsafe.putLong(allocateMemory, 72623859790382856L);
            switch (Bits.unsafe.getByte(allocateMemory)) {
                case 1: {
                    byteOrder = ByteOrder.BIG_ENDIAN;
                    break;
                }
                case 8: {
                    byteOrder = ByteOrder.LITTLE_ENDIAN;
                    break;
                }
                default: {
                    assert false;
                    byteOrder = null;
                    break;
                }
            }
        }
        finally {
            Bits.unsafe.freeMemory(allocateMemory);
        }
        Bits.pageSize = -1;
        Bits.unalignedKnown = false;
        Bits.maxMemory = VM.maxDirectMemory();
        reservedMemory = new AtomicLong();
        totalCapacity = new AtomicLong();
        count = new AtomicLong();
        Bits.memoryLimitSet = false;
        SharedSecrets.setJavaNioAccess(new JavaNioAccess() {
            public JavaNioAccess.BufferPool getDirectBufferPool() {
                return new JavaNioAccess.BufferPool() {
                    public String getName() {
                        return "direct";
                    }
                    
                    public long getCount() {
                        return Bits.count.get();
                    }
                    
                    public long getTotalCapacity() {
                        return Bits.totalCapacity.get();
                    }
                    
                    public long getMemoryUsed() {
                        return Bits.reservedMemory.get();
                    }
                };
            }
            
            public ByteBuffer newDirectByteBuffer(long n, int n2, Object o) {
                return new DirectByteBuffer(n, n2, o);
            }
            
            public void truncate(Buffer buffer) {
                buffer.truncate();
            }
        });
    }
}
